/*
 *    Copyright (C) 2002  Claudio Agosti - vecna@s0ftpj.org
 *                                         claudio@blackhats.it
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/stat.h>
#include <string.h>
#include <fcntl.h>
#include <dlfcn.h>
#include <errno.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <stdarg.h>

#include "shared-func.h"

ssize_t read(int, void *, size_t);
ssize_t write(int, const void *, size_t);
static int check_umpf_mode(int);
static void *take_glibc_func(char *);

/* LIB_PATH must to be passed on gcc with -D option with "~" */

/* glibc func */
ssize_t read(int fd, void *buf, size_t nbytes)
	{
	ssize_t (*glibc_read)(int, void *, size_t);
	off_t (*glibc_lseek)(int, off_t, int);
	ssize_t ret;

	(void *)glibc_lseek =take_glibc_func("lseek");
	(void *)glibc_read =take_glibc_func("read");

	if(check_umpf_mode(fd))
		{
		off_t offset;

		offset =glibc_lseek(fd, (off_t)0, SEEK_CUR);
	
		ret =glibc_read(fd, buf, nbytes);

		umpf_dec_buf((char *)buf, (unsigned int)ret, (off_t)offset);
		}
	else
		ret =glibc_read(fd, buf, nbytes);

	return ret;
	}

ssize_t write(int fd, const void *buf, size_t nbytes)
	{
	ssize_t (*glibc_write)(int, const void *, size_t);
	off_t (*glibc_lseek)(int, off_t, int);
	ssize_t ret;

	(void *)glibc_lseek =take_glibc_func("lseek");
	(void *)glibc_write =take_glibc_func("write");

	if(check_umpf_mode(fd))
		{
		off_t offset;
	
		offset =glibc_lseek(fd, (off_t)0, SEEK_CUR);

		umpf_enc_buf((void *)buf, (unsigned int)nbytes, (off_t)offset);

		ret =glibc_write(fd, buf, nbytes);
		}
	else
		ret =glibc_write(fd, buf, nbytes);

	return ret;
	}

static int check_umpf_mode(int fd)
	{
	struct stat ss;
	int ret =0;
	
	if((fstat(fd, &ss)) ==-1)
		fatal(USE_ERRNO, "unable to fstat umpf file");
	else 
		if(ss.st_mode & S_ISVTX)
			ret++;

	return ret;
	}

/* function for get original function in the glibc shared library */
static void *take_glibc_func(char *function)
	{
	void *glibc_shl;

	glibc_shl =dlopen(LIB_PATH, 1);

	if(glibc_shl ==NULL)
		fatal(USE_ERRNO, "umpf: fatal error on dlopen!");

	return(dlsym(glibc_shl, function));
	}
